//============================================================================
// Name        : LLHT.cpp
// Author      : Majid Kazemian
// Email       : akazemi2@uiuc.edu
// Copyright   : 
//============================================================================

#include "LLHT.h"
#include <iostream>
using namespace std;
#include "LLHTDist.h"

bool use_cond_geneset =false; //for conditional geneset
bool use_uniform_weights =false; //for non-weighted HG (traditional HG) this value equals to true
char* cond_geneset;

int main(int argc, char * argv []) {
	Process_Options (argc, argv);
	if(!use_cond_geneset){
		LLHTDist hg((char*)argv[argc-3],(char*)argv[argc-2],(char*)argv[argc-1], NULL ,use_uniform_weights);
		hg.get_significance();
	}else if(use_cond_geneset){
		LLHTDist hg((char*)argv[argc-3],(char*)argv[argc-2],(char*)argv[argc-1], cond_geneset, use_uniform_weights);
		hg.get_cond_significance();
	}
	return 0;
}

void  Process_Options (int argc, char * argv []){
//  Get options and parameters from command line.

   int  errflg = false;
   int  ch;

   optarg = NULL;
   while(! errflg && ((ch = getopt (argc, argv, "c:uh")) != EOF)){
     switch(ch){
		 case 'c':
			 use_cond_geneset =true;
			 cond_geneset = optarg;
    	 break;
		 case 'u':
			 use_uniform_weights = true;
		 break;
		 case 'h':
			 errflg =true;
			 break;
		 case '?':
			 fprintf (stderr, "Unrecognized option -%c\n", optopt);
		 default :
			 errflg = true;
     }
   }
   if(errflg || argc < 4 || argc< optind + 3){
        Usage ();
        exit (EXIT_FAILURE);
   }
   return;
}


static void  Usage (){
//  Print the description of options and command line for this program.

   fprintf (stderr,
		   "USAGE:  LLHT [options] <Universe> <Geneset> <sample>\n"
		   "Reads the sample file, and calculates the non central hyper-geometric test.\n"
		   "The test takes to account the non-uniform weights on Universe.\n"
		   "Options:\n"
		   "-c <Geneset> the geneset that HG test is conditioned on \n"
		   "-u apply uniform weights on all genes\n"

   );
   return;
}

