/*
 * LLHTDist.cpp
 *
 *  Created on: Jun 21, 2010
 *      Author: Majid Kazemian
 *      Email: akazemi2@uiuc.edu
 */

#include "LLHTDist.h"

LLHTDist::LLHTDist(char* universe_file, char* geneset_file, char* sample_file, char* cond_geneset, bool use_uniform_weights){
	// read the universe file and fill universe_map
	char* p;
	char line [MAX_LINE];
	FILE* fp = fopen(universe_file,"r");
	if (fp == NULL) {
		cout<<"Error reading Universe file"<<universe_file<<endl;
		cout<<"File should have two columns (gene name and weight)"<<endl;
		exit(1);
	}
	while(fgets(line, MAX_LINE, fp)!=NULL){
		int idx=0;
		string cg;
		long double wt=0;
		p = strtok (line, " \t\n\r");
		while (p != NULL){
			if (idx==0) cg = p;
			if (idx==1) {wt = atof(p); break;}
			p = strtok (NULL, " \t\n\r");
			idx++;
		}
		universe_map.insert(pair<string,long double>(cg,wt));
	}
	fclose(fp);

	//convert the second column to probability
	map<string,long double>::iterator it;
	long double sum=0;
	for (it=universe_map.begin(); it != universe_map.end(); it++){
       	sum+=(long double)it->second;
	}
	for (it=universe_map.begin(); it != universe_map.end(); it++){
		if(use_uniform_weights){
			it->second = (long double)1.0/universe_map.size();
		}else{
			it->second/=sum;
		}
	}

	//read the geneset file and fill geneset_vec
	fp = fopen(geneset_file,"r");
	if (fp == NULL) {
		cout<<"Error reading Geneset file"<<geneset_file<<endl;
		cout<<"File should have one column (gene names)"<<endl;
		exit(1);
	}
	while(fgets(line, MAX_LINE, fp)!=NULL){
		int idx=0;
		string cg;
		p = strtok (line, " \t\n");
		while (p != NULL){
			if (idx==0) {cg = p; break;}
			p = strtok (NULL, " \t\n");
			idx++;
		}
        map<string,long double>::iterator it;
        it=universe_map.find(cg);
        if(it!=universe_map.end()){
        	geneset_map.insert(pair<string,long double>(cg,(long double)it->second));
        }
	}
	fclose(fp);

	// read the sample file and fill sample_vec
	fp = fopen(sample_file,"r");
	if (fp == NULL) {
		cout<<"Error reading Sample file"<<sample_file<<endl;
		cout<<"File should have one column (gene names)"<<endl;
		exit(1);
	}
	while(fgets(line, MAX_LINE, fp)!=NULL){
		int idx=0;
		string cg;
		p = strtok (line, " \t\n");
		while (p != NULL){
			if (idx==0) {cg = p; break;}
			p = strtok (NULL, " \t\n");
			idx++;
		}
        map<string,long double>::iterator it;
        it=universe_map.find(cg);
        if(it!=universe_map.end()){
        	sample_map.insert(pair<string,long double>(cg,(long double)it->second));
        }
	}
	fclose(fp);

	// read conditional geneset file if it exists
	if(cond_geneset == NULL){return;}
	fp = fopen(cond_geneset,"r");
	if (fp == NULL) {
		cout<<"Error reading conditional geneset file"<<cond_geneset<<endl;
		cout<<"File should have one column (gene names)"<<endl;
		exit(1);
	}
	while(fgets(line, MAX_LINE, fp)!=NULL){
		int idx=0;
		string cg;
		p = strtok (line, " \t\n");
		while (p != NULL){
			if (idx==0) {cg = p; break;}
			p = strtok (NULL, " \t\n");
			idx++;
		}
        map<string,long double>::iterator it;
        it=universe_map.find(cg);
        if(it!=universe_map.end()){
        	cond_gs_map.insert(pair<string,long double>(cg,(long double)it->second));
        }
	}
	fclose(fp);
}
LLHTDist::~LLHTDist(){

}
long double LLHTDist::Sample_prob(map<string, long double> in_map, int sample_size, long double **dp_table){
	// allocate DP table
	int nrows = (int)in_map.size()+1;
	int ncols = sample_size+1;

	//dp_table = new long double* [nrows];
	for (int i = 0; i <nrows; i++)
		dp_table[i] = new long double[ncols];

	// initialiaze dp_table
	for (int i = 0; i <nrows; i++){
		for (int j = 0; j <ncols; j++){
			if(j==0){
				dp_table[i][j]=0;
			}else{
				dp_table[i][j]=NEGINF;
			}
		}
	}

	// fill DP table values
    map<string,long double>::iterator it;
    int i=1;
	for (it=in_map.begin(), i=1; it != in_map.end(); i++, it++){
		long double pl = (long double)it->second;
		for (int j = 1; j <ncols && j<=i; j++){
			dp_table[i][j]=LogSum(dp_table[i-1][j],dp_table[i-1][j-1]+log(pl));
			//dp_table[i][j]=dp_table[i-1][j]+dp_table[i-1][j-1]*pl;
			//cerr << "dp_table" << i<< " " << j << " "<<dp_table[i][j]<<endl;
		}
	}

	return dp_table[nrows-1][ncols-1];
}

long double LLHTDist::get_cond_significance(){

    // find the geneset minus cond_gs
	map<string,long double> gs_M_condgs_map; //geneset minus cond_gs
	map<string,long double>::iterator it;
	for (it=geneset_map.begin(); it != geneset_map.end(); it++){
		string cg = it->first;
		map<string,long double>::iterator it1;
		it1 = cond_gs_map.find(cg);
		if(it1==cond_gs_map.end()){
        	gs_M_condgs_map.insert(pair<string,long double>(cg,(long double)it->second));
		}
	}

    // find the universe minus cond_gs
	map<string,long double> uni_M_condgs_map; //universe minus cond_gs
	for (it=universe_map.begin(); it != universe_map.end(); it++){
		string cg = it->first;
		map<string,long double>::iterator it1;
		it1 = cond_gs_map.find(cg);
		if(it1==cond_gs_map.end()){
        	uni_M_condgs_map.insert(pair<string,long double>(cg,(long double)it->second));
		}
	}

    // find the universe minus (geneset union cond_gs)
	map<string,long double> uni_M_gsUcondgs_map; //universe minus (geneset union cond_gs)
	for (it=uni_M_condgs_map.begin(); it != uni_M_condgs_map.end(); it++){
		string cg = it->first;
		map<string,long double>::iterator it1;
		it1 = geneset_map.find(cg);
		if(it1==geneset_map.end()){
			uni_M_gsUcondgs_map.insert(pair<string,long double>(cg,(long double)it->second));
		}
	}

    // find the size of overlap between cond_gs and sample
    int alpha=0; //overlap size
	for (it=sample_map.begin(); it != sample_map.end(); it++){
		string cg = it->first;
		map<string,long double>::iterator it1;
		it1 = cond_gs_map.find(cg);
		if(it1!=cond_gs_map.end()){
			alpha++;
		}
	}

    // find the size of overlap between cond_gs, sample and geneset
    int beta=0; //overlap size
	for (it=sample_map.begin(); it != sample_map.end(); it++){
		string cg = it->first;
		map<string,long double>::iterator it1;
		it1 = cond_gs_map.find(cg);
		map<string,long double>::iterator it2;
		it2 = geneset_map.find(cg);
		if(it1!=cond_gs_map.end() && it2 != geneset_map.end()){
			beta++;
		}
	}

    // find the size of overlap between geneset and sample
    int lambda=0; //overlap size
	for (it=sample_map.begin(); it != sample_map.end(); it++){
		string cg = it->first;
		map<string,long double>::iterator it1;
		it1 = geneset_map.find(cg);
		if(it1!=geneset_map.end()){
			lambda++;
		}
	}

	int N =universe_map.size();
	int n = sample_map.size();
	int m = geneset_map.size();
	int e = cond_gs_map.size();
	int gama = m-gs_M_condgs_map.size();

	long double **uniMcondgs_dp_table =new long double*[uni_M_condgs_map.size()+1];
	long double denom =  Sample_prob(uni_M_condgs_map, n-alpha , uniMcondgs_dp_table);

	long double **gsMcondgs_dp_table = new long double*[gs_M_condgs_map.size()+1];
	Sample_prob(gs_M_condgs_map, min(m,n), gsMcondgs_dp_table);

	long double **uniMgsUcondgs_dp_table = new long double* [uni_M_gsUcondgs_map.size()+1];
	Sample_prob(uni_M_gsUcondgs_map, n-alpha ,uniMgsUcondgs_dp_table);

	long double sum =NEGINF;
	for(int k=lambda-beta; k<=min((int)gs_M_condgs_map.size(),n-alpha); k++){
		//sum+=gs_dp_table[m][i]*umg_dp_table[N-m][n-i];
		sum = LogSum(sum,gsMcondgs_dp_table[gs_M_condgs_map.size()][k] + uniMgsUcondgs_dp_table[uni_M_gsUcondgs_map.size()][n-alpha-k] - denom);
	}
	long double retval = exp(sum);

//	cerr<<"(N,m,o,e,lambda, gama, alpha, beta) "<<N<<","<<m<<","<<n<<","<<e<<","<<lambda<<","<<gama<<","<<alpha<<","<<beta<<endl;
	cout << N <<","<< m <<","<< n <<","<< lambda <<","<<gama<<","<<alpha<<","<<beta<<","<<retval<<endl;

	// free DP table
	for (int i = 0; i <(int)uni_M_condgs_map.size(); i++){
		delete [] uniMcondgs_dp_table[i];
	}
	delete [] uniMcondgs_dp_table;

	for (int i = 0; i <(int)gs_M_condgs_map.size(); i++){
		delete [] gsMcondgs_dp_table[i];
	}
	delete [] gsMcondgs_dp_table;

	for (int i = 0; i <(int)uni_M_gsUcondgs_map.size(); i++){
		delete [] uniMgsUcondgs_dp_table[i];
	}
	delete [] uniMgsUcondgs_dp_table;


	return retval;

}

long double LLHTDist::get_significance(){

    // find the number of overlaps between geneset and sample
	map<string,long double>::iterator it;
    int k=0; //overlap size
	for (it=sample_map.begin(); it != sample_map.end(); it++){
		string cg = it->first;
		map<string,long double>::iterator it1;
		it1 = geneset_map.find(cg);
		if(it1!=geneset_map.end()){
			k++;
		}
	}

	int N = universe_map.size();
	int m = geneset_map.size();
	int n = sample_map.size();

	//create a new set as universe minus geneset
	map<string,long double> umg_map;
	for (it=universe_map.begin(); it != universe_map.end(); it++){
		string cg = it->first;
		map<string,long double>::iterator it1;
		it1 = geneset_map.find(cg);
		if(it1==geneset_map.end()){
        	umg_map.insert(pair<string,long double>(cg,(long double)it->second));
		}
	}

	long double **uni_dp_table =new long double*[N+1];
	long double denom =  Sample_prob(universe_map, n, uni_dp_table);

	long double **gs_dp_table = new long double*[m+1];
	long double **umg_dp_table = new long double* [N-m+1];
	Sample_prob(geneset_map, m, gs_dp_table);
	Sample_prob(umg_map, n,umg_dp_table);


	long double sum =NEGINF;
	for(int i=k; i<=min(m,n); i++){
		//sum+=gs_dp_table[m][i]*umg_dp_table[N-m][n-i];
		sum = LogSum(sum,gs_dp_table[m][i]+umg_dp_table[N-m][n-i]-denom);
	}
	long double retval = exp(sum);

//	cerr << N <<","<< m <<","<< n <<","<< k <<","<<retval<<endl;
	cout << N <<","<< m <<","<< n <<","<< k <<","<< "-" <<","<< "-" <<","<< "-"<<","<<retval<<endl;

	// free DP table
	for (int i = 0; i <N; i++){
		delete [] uni_dp_table[i];
	}
	delete [] uni_dp_table;
	for (int i = 0; i <m; i++){
		delete [] gs_dp_table[i];
	}
	delete [] gs_dp_table;
	for (int i = 0; i <N-m; i++){
		delete [] umg_dp_table[i];
	}
	delete [] umg_dp_table;

	return retval;
}

long double LLHTDist::LogSum (long double x, long double y)
{
  if (x <= NEGINF) {
    if (y <= NEGINF) return NEGINF;
    else return y;
  }
  else {
    if (y <= NEGINF) return x;
    else {
      if (y >= x) return y + log(1.0 + exp(x-y));
      return x + log(1.0 +exp(y-x));
    }
  }
}

